@@ -1320,6 +1320,17 @@ totem_embedded_toggle_fullscreen (TotemEmbedded *emb)
 		
 		gtk_action_set_sensitive (fs_action, TRUE);
 	} else {
+		GdkRectangle rect;
+		int monitor;
+
+		/* Move the fullscreen window to the screen where the
+		 * video widget currently is */
+		monitor = gdk_screen_get_monitor_at_window (gtk_widget_get_screen (GTK_WIDGET (emb->bvw)),
+							    GTK_WIDGET (emb->bvw)->window);
+		gdk_screen_get_monitor_geometry (gtk_widget_get_screen (GTK_WIDGET (emb->bvw)),
+						 monitor, &rect);
+		gtk_window_move (GTK_WINDOW (emb->fs_window), rect.x, rect.y);
+
 		gtk_widget_reparent (GTK_WIDGET (emb->bvw), emb->fs_window);
 		bacon_video_widget_set_fullscreen (emb->bvw, TRUE);
 		gtk_window_fullscreen (GTK_WINDOW (emb->fs_window));
@@ -1990,6 +2001,7 @@ entry_parsed (TotemPlParser *parser,
 {
 	TotemEmbedded *emb = (TotemEmbedded *) data;
 	TotemPlItem *item;
+	int duration;
 
 	if (g_str_has_prefix (uri, "file://") != FALSE
 	    && g_str_has_prefix (emb->base_uri, "file://") == FALSE) {
@@ -2000,9 +2012,14 @@ entry_parsed (TotemPlParser *parser,
 	g_print ("added URI '%s'\n", uri);
 	g_hash_table_foreach (metadata, (GHFunc) entry_metadata_foreach, NULL);
 
+	/* Skip short advert streams */
+	duration = totem_embedded_parse_duration (g_hash_table_lookup (metadata, "duration"));
+	if (duration == 0)
+		return;
+
 	item = g_new0 (TotemPlItem, 1);
 	item->uri = g_strdup (uri);
-	item->duration = totem_embedded_parse_duration (g_hash_table_lookup (metadata, "duration"));
+	item->duration = duration;
 	item->starttime = totem_embedded_parse_duration (g_hash_table_lookup (metadata, "starttime"));
 
 	emb->playlist = g_list_prepend (emb->playlist, item);
