@@ -207,6 +207,24 @@ totem_playlist_get_toplevel (TotemPlaylist *playlist)
 	return GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (playlist)));
 }
 
+static void
+totem_playlist_set_waiting_cursor (TotemPlaylist *playlist)
+{
+	GtkWidget *parent;
+
+	parent = GTK_WIDGET (totem_playlist_get_toplevel (playlist));
+	totem_gdk_window_set_waiting_cursor (parent->window);
+}
+
+static void
+totem_playlist_unset_waiting_cursor (TotemPlaylist *playlist)
+{
+	GtkWidget *parent;
+
+	parent = GTK_WIDGET (totem_playlist_get_toplevel (playlist));
+	gdk_window_set_cursor (parent->window, NULL);
+}
+
 static void
 totem_playlist_error (char *title, char *reason, TotemPlaylist *playlist)
 {
@@ -377,6 +395,8 @@ drop_cb (GtkWidget     *widget,
 		return;
 	}
 
+	totem_playlist_set_waiting_cursor (playlist);
+
 	for (p = file_list; p != NULL; p = p->next)
 	{
 		char *filename, *title;
@@ -385,6 +405,8 @@ drop_cb (GtkWidget     *widget,
 			continue;
 
 		filename = totem_create_full_path (p->data);
+		if (filename == NULL)
+			filename = g_strdup (p->data);
 		title = NULL;
 
 		if (info == 1) {
@@ -398,7 +420,7 @@ drop_cb (GtkWidget     *widget,
 			}
 		}
 
-		totem_playlist_add_mrl (playlist, filename ? filename : p->data, title);
+		totem_playlist_add_mrl (playlist, filename, title);
 
 		g_free (filename);
 		g_free (p->data);
@@ -407,6 +429,8 @@ drop_cb (GtkWidget     *widget,
 	g_list_free (file_list);
 	gtk_drag_finish (context, TRUE, FALSE, time);
 
+	totem_playlist_unset_waiting_cursor (playlist);
+
 	g_signal_emit (G_OBJECT (playlist),
 			totem_playlist_table_signals[CHANGED], 0,
 			NULL);
@@ -685,6 +709,8 @@ totem_playlist_add_files (GtkWidget *widget, TotemPlaylist *playlist)
 	if (filenames == NULL)
 		return;
 
+	totem_playlist_set_waiting_cursor (playlist);
+
 	for (l = filenames; l != NULL; l = l->next) {
 		char *mrl;
 
@@ -693,6 +719,7 @@ totem_playlist_add_files (GtkWidget *widget, TotemPlaylist *playlist)
 		g_free (mrl);
 	}
 
+	totem_playlist_unset_waiting_cursor (playlist);
 	g_slist_free (filenames);
 }
 
@@ -1409,6 +1436,39 @@ init_config (TotemPlaylist *playlist)
 			GCONF_PREFIX"/shuffle", NULL) != FALSE;
 }
 
+/* FIXME this should live in the playlist parser */
+static int
+totem_embedded_parse_duration (const char *duration)
+{
+	int hours, minutes, seconds, fractions;
+
+	if (!duration)
+		return -1;
+
+	/* Formats used by both ASX and RAM files */
+	if (sscanf (duration, "%d:%d:%d.%d", &hours, &minutes, &seconds, &fractions) == 4) {
+		int ret = hours * 3600 + minutes * 60 + seconds;
+		if (ret == 0 && fractions > 0)
+			ret = 1;
+		return ret;
+	}
+	if (sscanf (duration, "%d:%d:%d", &hours, &minutes, &seconds) == 3)
+		return hours * 3600 + minutes * 60 + seconds;
+	if (sscanf (duration, "%d:%d.%d", &minutes, &seconds, &fractions) == 3) {
+		int ret = minutes * 60 + seconds;
+		if (ret == 0 && fractions > 0)
+			ret = 1;
+		return ret;
+	}
+	if (sscanf (duration, "%d:%d", &minutes, &seconds) == 2)
+		return minutes * 60 + seconds;
+	/* PLS files format */
+	if (sscanf (duration, "%d", &seconds) == 1)
+		return seconds;
+
+	return -1;
+}
+
 static void
 totem_playlist_entry_parsed (TotemPlParser *parser,
 			     const char *uri,
@@ -1416,7 +1476,13 @@ totem_playlist_entry_parsed (TotemPlParser *parser,
 			     TotemPlaylist *playlist)
 {
 	const char *title;
+	gint64 duration;
 
+	/* We ignore 0-length items in playlists, they're usually just banners */
+	duration = totem_embedded_parse_duration
+		(g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_DURATION));
+	if (duration == 0)
+		return;
 	title = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_TITLE);
 	totem_playlist_add_one_mrl (playlist, uri, title);
 }
@@ -1575,25 +1641,38 @@ totem_playlist_add_one_mrl (TotemPlaylist *playlist, const char *mrl,
 	return TRUE;
 }
 
+gboolean
+totem_playlist_add_mrl_with_cursor (TotemPlaylist *playlist, const char *mrl,
+				    const char *display_name)
+{
+	gboolean retval;
+
+	totem_playlist_set_waiting_cursor (playlist);
+	retval = totem_playlist_add_mrl (playlist, mrl, display_name);
+	totem_playlist_unset_waiting_cursor (playlist);
+
+	return retval;
+}
+
 gboolean
 totem_playlist_add_mrl (TotemPlaylist *playlist, const char *mrl,
-		const char *display_name)
+			const char *display_name)
 {
 	TotemPlParserResult res;
-	GtkWidget *parent;
 
 	g_return_val_if_fail (mrl != NULL, FALSE);
 
-	parent = GTK_WIDGET (totem_playlist_get_toplevel (playlist));
-	totem_gdk_window_set_waiting_cursor (parent->window);
-	res = totem_pl_parser_parse (playlist->_priv->parser, mrl, TRUE);
-	gdk_window_set_cursor (parent->window, NULL);
+	res = totem_pl_parser_parse (playlist->_priv->parser, mrl, FALSE);
 
 	if (res == TOTEM_PL_PARSER_RESULT_UNHANDLED)
 		return totem_playlist_add_one_mrl (playlist, mrl, display_name);
 	if (res == TOTEM_PL_PARSER_RESULT_ERROR)
 	{
-		totem_playlist_error (_("Playlist error"), _("The playlist '%s' could not be parsed, it might be damaged."), playlist);
+		char *msg;
+
+		msg = g_strdup_printf (_("The playlist '%s' could not be parsed, it might be damaged."), display_name ? display_name : mrl);
+		totem_playlist_error (_("Playlist error"), msg, playlist);
+		g_free (msg);
 		return FALSE;
 	}
 	if (res == TOTEM_PL_PARSER_RESULT_IGNORED)
