@@ -75,8 +75,12 @@ GtkWidget *totem_playlist_new      (void);
  * @display_name is if you have a preferred display string for the mrl,
  * NULL otherwise
  */
-gboolean totem_playlist_add_mrl  (TotemPlaylist *playlist, const char *mrl,
+gboolean totem_playlist_add_mrl  (TotemPlaylist *playlist,
+				  const char *mrl,
 				  const char *display_name);
+gboolean totem_playlist_add_mrl_with_cursor (TotemPlaylist *playlist,
+					     const char *mrl,
+					     const char *display_name);
 
 void totem_playlist_save_current_playlist (TotemPlaylist *playlist,
 					   const char *output);
@@ -93,6 +97,9 @@ void       totem_playlist_clear_with_gnome_vfs_volume (TotemPlaylist *playlist,
 char      *totem_playlist_get_current_mrl (TotemPlaylist *playlist);
 char      *totem_playlist_get_current_title (TotemPlaylist *playlist,
 					     gboolean *custom);
+char      *totem_playlist_get_title (TotemPlaylist *playlist,
+				     guint index);
+
 gboolean   totem_playlist_get_current_metadata (TotemPlaylist *playlist,
 						char         **artist,
 						char         **title,
